// @ts-check

/**
 * ==============================================================================
 * Imports
 * ==============================================================================
 */
const http = require("http");
const https = require("https");
const fs = require("fs");
const path = require("path");
const grabHostNames = require("../package-shared/utils/grab-host-names");
const apiSendEmailCode = require("../package-shared/functions/api/users/api-send-email-code");

/** ****************************************************************************** */
/** ****************************************************************************** */
/** ****************************************************************************** */
/** ****************************************************************************** */
/** ****************************************************************************** */
/** ****************************************************************************** */

/**
 * Send Email Code to a User
 * ==============================================================================
 * @async
 *
 * @param {object} params - Single Param object containing params
 * @param {String} [params.key] - FULL ACCESS API Key
 * @param {String} [params.database] - Target Database
 * @param {string} params.email Login Email/Username and Password
 * @param {http.ServerResponse} [params.response] - Http response object
 * @param {string} [params.temp_code_field_name] - Database table field name for temporary code
 * @param {string} [params.mail_domain]
 * @param {string} [params.mail_username]
 * @param {string} [params.mail_password]
 * @param {number} [params.mail_port]
 * @param {string} [params.sender]
 * @param {boolean} [params.user_id] - User ID
 * @param {boolean} [params.useLocal]
 *
 * @returns { Promise<boolean>}
 */
async function sendEmailCode({
    key,
    email,
    database,
    temp_code_field_name,
    mail_domain,
    mail_password,
    mail_username,
    mail_port,
    sender,
    user_id,
    useLocal,
}) {
    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    const defaultTempLoginFieldName = "temp_login_code";
    const emailLoginTempCodeFieldName = temp_code_field_name
        ? temp_code_field_name
        : defaultTempLoginFieldName;

    const emailHtml = fs.readFileSync(
        path.resolve(__dirname, "../package-shared/html/one-time-code.html"),
        "utf-8"
    );

    /**
     * Initialize HTTP response variable
     */
    let httpResponse;

    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } =
        process.env;

    if (
        DSQL_DB_HOST?.match(/./) &&
        DSQL_DB_USERNAME?.match(/./) &&
        DSQL_DB_PASSWORD?.match(/./) &&
        DSQL_DB_NAME?.match(/./) &&
        useLocal
    ) {
        /** @type {import("../package-shared/types").DSQL_DatabaseSchemaType | undefined} */
        let dbSchema;

        try {
            const localDbSchemaPath = path.resolve(
                process.cwd(),
                "dsql.schema.json"
            );
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        } catch (error) {}

        httpResponse = await apiSendEmailCode({
            database: DSQL_DB_NAME,
            email,
            email_login_field: emailLoginTempCodeFieldName,
            html: emailHtml,
            mail_domain,
            mail_password,
            mail_port,
            mail_username,
            sender,
            useLocal,
        });
    } else {
        /**
         * Make https request
         *
         * @description make a request to datasquirel.com
         *
         * @type {{ success: boolean, payload: import("../package-shared/types").DATASQUIREL_LoggedInUser | null, userId?: number, msg?: string }}
         */
        httpResponse = await new Promise((resolve, reject) => {
            const reqPayload = JSON.stringify({
                email,
                database,
                email_login_field: emailLoginTempCodeFieldName,
                mail_domain,
                mail_password,
                mail_username,
                mail_port,
                sender,
                html: emailHtml,
            });

            const httpsRequest = scheme.request(
                {
                    method: "POST",
                    headers: {
                        "Content-Type": "application/json",
                        "Content-Length": Buffer.from(reqPayload).length,
                        Authorization:
                            key ||
                            process.env.DSQL_FULL_ACCESS_API_KEY ||
                            process.env.DSQL_API_KEY,
                    },
                    port,
                    hostname: host,
                    path: `/api/user/${
                        user_id || grabedHostNames.user_id
                    }/send-email-code`,
                },

                /**
                 * Callback Function
                 *
                 * @description https request callback
                 */
                (res) => {
                    var str = "";

                    res.on("data", function (chunk) {
                        str += chunk;
                    });

                    res.on("end", function () {
                        resolve(JSON.parse(str));
                    });

                    res.on("error", (err) => {
                        reject(err);
                    });
                }
            );

            httpsRequest.write(reqPayload);
            httpsRequest.end();
        });
    }

    /** ********************************************** */
    /** ********************************************** */
    /** ********************************************** */

    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    if (httpResponse?.success) {
        return true;
    } else {
        console.log(httpResponse);
        return false;
    }
}

/** ********************************************** */
/** ********************************************** */
/** ********************************************** */

module.exports = sendEmailCode;
