// @ts-check

const https = require("node:https");
const path = require("path");
const fs = require("fs");
const serializeQuery = require("./functions/serialize-query");
const grabHostNames = require("../package-shared/utils/grab-host-names");
const apiGet = require("../package-shared/functions/api/query/get");

/**
 * Make a get request to Datasquirel API
 * ==============================================================================
 * @async
 *
 * @param {Object} params - Single object passed
 * @param {string} [params.key] - API Key
 * @param {string} [params.db] - Database Name
 * @param {string} params.query - SQL Query
 * @param {string[]} [params.queryValues] - An array of query values if using "?" placeholders
 * @param {string} [params.tableName] - Name of the table to query
 * @param {boolean} [params.useLocal] - Whether to use a remote database instead of API
 * @param {string | number} [params.user_id] - User ID
 *
 * @returns { Promise<import("../package-shared/types").GetReturn> } - Return Object
 */
async function get({
    key,
    db,
    query,
    queryValues,
    tableName,
    useLocal,
    user_id,
}) {
    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } =
        process.env;

    if (
        DSQL_DB_HOST?.match(/./) &&
        DSQL_DB_USERNAME?.match(/./) &&
        DSQL_DB_PASSWORD?.match(/./) &&
        DSQL_DB_NAME?.match(/./) &&
        useLocal
    ) {
        /** @type {import("../package-shared/types").DSQL_DatabaseSchemaType | undefined} */
        let dbSchema;

        try {
            const localDbSchemaPath = path.resolve(
                process.cwd(),
                "dsql.schema.json"
            );
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        } catch (error) {}

        return await apiGet({
            dbFullName: DSQL_DB_NAME,
            query,
            queryValues,
            tableName,
            dbSchema,
            useLocal,
        });
    }

    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    const httpResponse = await new Promise((resolve, reject) => {
        /** @type {import("../package-shared/types").GetReqQueryObject} */
        const queryObject = {
            db: process.env.DSQL_API_DB_NAME || String(db),
            query: String(
                query
                    .replace(/\n|\r|\n\r/g, "")
                    .replace(/ {2,}/g, " ")
                    .replace(/ /g, "+")
            ),
            queryValues: queryValues ? JSON.stringify(queryValues) : undefined,
            tableName,
        };

        const queryString = serializeQuery({ query: queryObject });

        let path = `/api/query/${
            user_id || grabedHostNames.user_id
        }/get${queryString}`;

        /** @type {https.RequestOptions} */
        const requestObject = {
            method: "GET",
            headers: {
                "Content-Type": "application/json",
                Authorization:
                    key ||
                    process.env.DSQL_READ_ONLY_API_KEY ||
                    process.env.DSQL_FULL_ACCESS_API_KEY ||
                    process.env.DSQL_API_KEY,
            },
            port,
            hostname: host,
            path: encodeURI(path),
        };

        console.log("requestObject", requestObject);

        scheme
            .request(
                requestObject,

                /**
                 * Callback Function
                 *
                 * @description https request callback
                 */
                (response) => {
                    var str = "";

                    response.on("data", function (chunk) {
                        str += chunk;
                    });

                    response.on("end", function () {
                        try {
                            resolve(JSON.parse(str));
                        } catch (/** @type {any} */ error) {
                            reject({
                                error: error.message,
                                result: str,
                            });
                        }
                    });

                    response.on("error", (err) => {
                        console.log("DSQL get Error,", err.message);
                        reject(err);
                    });
                }
            )
            .end();
    });

    /** ********************************************** */
    /** ********************************************** */
    /** ********************************************** */

    return httpResponse;
}

/** ********************************************** */
/** ********************************************** */
/** ********************************************** */

module.exports = get;
